<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\MAN\Services\RelTempoPermanenciaMesaService;
use \Zeedhi\Framework\Util\Functions;

class RelTempoPermanenciaMesa extends CrudWithParameterBag {
    const dataSourceConsumidor  = 'orm00025_relTempoPermanenciaMesa';
    
    protected $relTempoPermanenciaMesaService;
    protected $parameterBagColumnsConsumidor = array('CDFILIAL', 'TCDLOJA', 'CDLOJA', 'DTINI', 'DTFIM', 'NRMESA', 'TNRMESA');
    
    public function __construct ($parameterBag, Manager $dataSourceManager, $utilService, RelTempoPermanenciaMesaService $relTempoPermanenciaMesaService) {
        $this->parameterBag                   = $parameterBag;
        $this->dataSourceManager              = $dataSourceManager;
        $this->utilService                    = $utilService;
        $this->relTempoPermanenciaMesaService = $relTempoPermanenciaMesaService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];

        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);

        if($this->validaConditions($conditions)) {
            $result = $this->relTempoPermanenciaMesaService->find($conditions);
            
            $response->addDataSet(new DataSet("orm00025_relTempoPermanenciaMesa", $result));
        }
        
        return $result;
    }

    private function buildParams($conditions) {
        $result = [];
        
        if(!Functions::arrayKeyExists('CDFILIAL', $conditions)) {
            $result['P_TCDFILIAL'] = 'T';
            $result['P_CDFILIAL'] = 'NULL';
        } else {
            $result['P_CDFILIAL'] = $conditions['CDFILIAL'];
            $result['P_TCDFILIAL'] = null;
        }

        if((Functions::arrayKeyExists('CDLOJA', $conditions)) && (is_array($conditions['CDLOJA']))) {
            $result['P_CDLOJA'] = "'" . implode("', '", $conditions['CDLOJA']) . "'";
            $result['P_TCDLOJA'] = null;
        } else {
            $result['P_TCDLOJA'] = 'T';
            $result['P_CDLOJA'] = 'NULL';
        }

        if(Functions::arrayKeyExists('DTENTRVENDA', $conditions)) {
            $result['P_DTINICIAL'] = $conditions['DTENTRVENDA'][0];
            $result['P_DTFINAL']   = $conditions['DTENTRVENDA'][1];
        }

        if(Functions::arrayKeyExists('NRMESA', $conditions)) {
            $result['P_NRMESA']  = "'" . implode("', '", $conditions['NRMESA']) . "'";
            $result['P_TNRMESA'] = NULL;
        } else {
            $result['P_NRMESA']  = 'NULL';
            $result['P_TNRMESA'] = 'T';
        }

        return $result;
    }

    private function validaConditions($conditions) {
        if((isset($conditions['P_CDFILIAL']))  && (!empty($conditions['P_CDFILIAL']))  &&
           (isset($conditions['P_DTINICIAL'])) && (!empty($conditions['P_DTINICIAL'])) &&
           (isset($conditions['P_DTFINAL']))   && (!empty($conditions['P_DTFINAL']))) {
            return true;
        } else {
            return false;
        }
    }
    
    public function findOld (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceConsumidor;
        $this->parameterBagColumns = $this->parameterBagColumnsConsumidor;
        $conditions                = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter          = self::prepareParameters($conditions);
        $requestDataSourceFilter = $request->getFilterCriteria()->getConditions();
        
        foreach($requestDataSourceFilter as $filter) {
            if($filter['operator'] === 'MAPPED_LIKE_ALL') {
                array_push($dataSourceFilter, $filter);
            }
        }
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $this->dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        
        parent::find($newRequest, $response);
    }
    
    private function prepareParameters($conditions) {
        if(isset($conditions['DTENTRVENDA'])) {
            $DTINI = $conditions['DTENTRVENDA'][0];
            $DTFIM = $conditions['DTENTRVENDA'][1];
        } else {
            $DTFIM = $DTINI = NULL;
        }
        if(isset($conditions['CDFILIAL'])){
            $CDFILIAL = $conditions['CDFILIAL'];
            $TCDFILIAL = NULL;
            
        } else {
            $CDFILIAL = NULL;
            $TCDFILIAL = 'T';
        }
        
        if(isset($conditions['CDLOJA']) && !!$conditions['CDLOJA']){
            $CDLOJA = implode('_', $conditions['CDLOJA']);
            $TCDLOJA = NULL;
            
        } else {
            $CDLOJA = NULL;
            $TCDLOJA = 'T';
        }
        
        if(isset($conditions['NRMESA']) && !!$conditions['NRMESA']){
            $NRMESA = implode('_', $conditions['NRMESA']);
            $TNRMESA = NULL;
            
        } else {
            $NRMESA = NULL;
            $TNRMESA = 'T';
        }
        
        $parameters = array(
            'CDFILIAL'  => $CDFILIAL,
            'TCDLOJA'   => $TCDLOJA,
            'CDLOJA'    => $CDLOJA,
            'NRMESA'    => $NRMESA,
            'TNRMESA'   => $TNRMESA,
            'DTINI'     => $DTINI,
            'DTFIM'     => $DTFIM
        );
        
        $dataSourceFilter = array();
        
        $dataSourceFilter[0] = array(
            "columnName" => 'CDFILIAL',
            "operator"   => "=",
            "value"      => $parameters['CDFILIAL']
        ); 
        $dataSourceFilter[1] = array(
            "columnName" => 'TCDLOJA',
            "operator"   => "=",
            "value"      => $parameters['TCDLOJA']
        );
        $dataSourceFilter[2] = array(
            "columnName" => 'CDLOJA',
            "operator"   => "=",
            "value"      => $parameters['CDLOJA']
        );
        $dataSourceFilter[3] = array(
            "columnName" => 'NRMESA',
            "operator"   => "=",
            "value"      => $parameters['NRMESA']
        ); 
        $dataSourceFilter[4] = array(
            "columnName" => 'TNRMESA',
            "operator"   => "=",
            "value"      => $parameters['TNRMESA']
        );
        $dataSourceFilter[5] = array(
            "columnName" => 'DTINI',
            "operator"   => "=",
            "value"      => $parameters['DTINI']
        );
        $dataSourceFilter[6] = array(
            "columnName" => 'DTFIM',
            "operator"   => "=",
            "value"      => $parameters['DTFIM']
        );
        
        return $dataSourceFilter;
    }
}